@@ -3,6 +3,8 @@ from unicrypto import hashlib
 import collections
 import datetime
 import secrets
+import os
+import io
 from typing import List
 
 from minikerberos import logger
@@ -12,14 +14,17 @@ from minikerberos.protocol.asn1_structs import METHOD_DATA, ETYPE_INFO, ETYPE_IN
 	PADATA_TYPE, PA_PAC_REQUEST, PA_ENC_TS_ENC, EncryptedData, krb5_pvno, KDC_REQ_BODY, \
 	AS_REQ, TGS_REP, KDCOptions, PrincipalName, EncASRepPart, EncTGSRepPart, PrincipalName, Realm, \
 	Checksum, APOptions, Authenticator, Ticket, AP_REQ, TGS_REQ, CKSUMTYPE, \
-	PA_FOR_USER_ENC, PA_PAC_OPTIONS, PA_PAC_OPTIONSTypes, EncTicketPart
+	PA_FOR_USER_ENC, PA_PAC_OPTIONS, PA_PAC_OPTIONSTypes, EncTicketPart, \
+	ChangePasswdDataMS, EncryptionKey, EncKrbPrivPart, HostAddress, KRB_PRIV,\
+	AP_REP
 
+from minikerberos.protocol.rfc3244 import KRB5ChangePassword, KRB5CHPWReply, KRB5CHPWResultCode
 from minikerberos.protocol.errors import KerberosErrorCode, KerberosError
 from minikerberos.protocol.encryption import Key, _enctype_table, _HMACMD5, Enctype, _checksum_table
 from minikerberos.protocol.constants import PaDataType, EncryptionType, NAME_TYPE, MESSAGE_TYPE
 from minikerberos.protocol.structures import AuthenticatorChecksum
 from minikerberos.gssapi.gssapi import GSSAPIFlags
-from minikerberos.network.aioclientsocket import AIOKerberosClientSocket
+from minikerberos.network.aioclientsocket import AIOKerberosClientSocket, AIOKerberosPWChangeClientSocket
 
 from minikerberos.common.creds import KerberosCredential
 from minikerberos.common.target import KerberosTarget
@@ -355,7 +360,7 @@ class AIOKerberosClient:
 		except Exception as e:
 			return None, None, None, e
 
-	async def get_TGS(self, spn_user:KerberosSPN, override_etype = None, is_linux = False):
+	async def get_TGS(self, spn_user:KerberosSPN, override_etype = None, is_linux = False, flags = ['forwardable','renewable','renewable_ok', 'canonicalize']):
 		"""
 		Requests a TGS ticket for the specified user.
 		Retruns the TGS ticket, end the decrpyted encTGSRepPart.
@@ -380,7 +385,7 @@ class AIOKerberosClient:
 		logger.debug('Constructing TGS request for user %s' % spn_user.get_formatted_pname())
 		now = datetime.datetime.now(datetime.timezone.utc)
 		kdc_req_body = {}
-		kdc_req_body['kdc-options'] = KDCOptions(set(['forwardable','renewable','renewable_ok', 'canonicalize']))
+		kdc_req_body['kdc-options'] = KDCOptions(set(flags))
 		kdc_req_body['realm'] = spn_user.domain.upper()
 		kdc_req_body['sname'] = PrincipalName({'name-type': NAME_TYPE.SRV_INST.value, 'name-string': spn_user.get_principalname()})
 		kdc_req_body['till'] = (now + datetime.timedelta(days=1)).replace(microsecond=0)
@@ -778,4 +783,106 @@ class AIOKerberosClient:
 		encasrep = EncASRepPart.load(dec_data).native
 		cipher = _enctype_table[ int(encasrep['key']['keytype'])]
 		session_key = Key(cipher.enctype, encasrep['key']['keyvalue'])
-		return encasrep, session_key, cipher
\ No newline at end of file
+		return encasrep, session_key, cipher
+	
+	async def get_referral_ticket(self, target_domain, target_ip = None):
+		"""Cross domain TGT referral"""
+		"""If target_ip is not set, the target domain will be used as the hostname for the newly created connection"""
+		from minikerberos.common.factory import KerberosClientFactory
+		from minikerberos.common.kirbi import Kirbi
+
+		crossrealm_spn = KerberosSPN()
+		crossrealm_spn.username = target_domain
+		crossrealm_spn.service = 'krbtgt'
+		crossrealm_spn.domain = self.credential.domain.upper()
+
+		await self.get_TGT()
+		logger.debug('Getting TGS for otherdomain krbtgt')
+		tgs, encpart, key = await self.get_TGS(crossrealm_spn)
+		logger.debug('Got referral ticket!')
+
+
+		kirbi = Kirbi.from_ticketdata(tgs, encpart)
+
+		target_addr = target_domain
+		if target_ip is not None:
+			target_addr = target_ip
+		newt = self.target.get_newtarget(target_addr, port=88)
+		newc = KerberosCredential.from_kirbi(kirbi, encoding='kirbi')
+		new_factory = KerberosClientFactory(newt, newc, newt.proxies)
+
+		return tgs, encpart, key, new_factory
+	
+	async def change_password(self, new_password:str, subkey = None, targetuser:str = None, targetrealm:str = None, hostname:str ='localhost') -> KRB5CHPWReply:
+		"""
+		Changes the password of the current user
+		"""
+		if not self.kerberos_TGT:
+			logger.debug('[ChangePassword] TGT is not available! Fetching TGT...')
+			await self.get_TGT()
+		
+		if subkey is None:
+			subkeydata = os.urandom(self.kerberos_cipher.keysize)
+			subkey = Key(self.kerberos_cipher.enctype, subkeydata)
+
+		now = datetime.datetime.now(datetime.timezone.utc)
+
+		###### Encrypting the new password
+		#enc_data = self.kerberos_cipher.encrypt(subkey, 5, new_password.encode(), None)
+		subkey_struct = EncryptionKey({'keytype': subkey.enctype, 'keyvalue': subkeydata})
+		subkey_cipher = _enctype_table[subkey.enctype]
+
+		changepwstruct = {}
+		changepwstruct['newpasswd'] = new_password.encode()
+
+		if targetuser is not None:
+			if targetuser.find('@') != -1:
+				targetuser, targetrealm = targetuser.split('@')
+			changepwstruct['targname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': [targetuser]})
+			changepwstruct['targrealm'] = targetrealm if targetrealm is not None else self.credential.domain.upper()
+		else:
+			changepwstruct['targname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': [self.credential.username]})
+			changepwstruct['targrealm'] = self.credential.domain.upper()
+
+		privstruct = {}
+		privstruct['user-data'] = ChangePasswdDataMS(changepwstruct).dump()
+		privstruct['seq-number'] = 0
+		privstruct['s-address'] = HostAddress({'addr-type': 1, 'address': hostname.encode()})
+
+		privdata = KRB_PRIV({
+			'pvno': krb5_pvno,
+			'msg-type': MESSAGE_TYPE.KRB_PRIV.value,
+			'enc-part': EncryptedData({'etype': subkey.enctype, 'cipher': subkey_cipher.encrypt(subkey, 13, EncKrbPrivPart(privstruct).dump(), None)})
+		}).dump()
+		
+		###### Calculating authenticator data
+		authenticator_data = {}
+		authenticator_data['authenticator-vno'] = krb5_pvno
+		authenticator_data['crealm'] = Realm(self.kerberos_TGT['crealm'])
+		authenticator_data['cname'] = self.kerberos_TGT['cname']
+		authenticator_data['cusec'] = now.microsecond
+		authenticator_data['ctime'] = now.replace(microsecond=0)
+		authenticator_data['seq-number'] = 0
+		authenticator_data['subkey'] = subkey_struct
+		
+		authenticator_data_enc = self.kerberos_cipher.encrypt(self.kerberos_session_key, 11, Authenticator(authenticator_data).dump(), None)
+		
+		ap_req = {}
+		ap_req['pvno'] = krb5_pvno
+		ap_req['msg-type'] = MESSAGE_TYPE.KRB_AP_REQ.value
+		ap_req['ap-options'] = APOptions(set())
+		ap_req['ticket'] = Ticket(self.kerberos_TGT['ticket'])
+		ap_req['authenticator'] = EncryptedData({'etype': self.kerberos_cipher_type, 'cipher': authenticator_data_enc})
+		
+		ap_req_encoded = AP_REQ(ap_req).dump()
+		
+		
+		message = KRB5ChangePassword(ap_req_encoded, privdata).to_bytes()
+		logger.debug('[ChangePassword] Sending request to server')
+
+		newt = self.target.get_newtarget(self.target.get_hostname_or_ip(), port=464)
+		ksocket = AIOKerberosPWChangeClientSocket(newt)
+		response = await ksocket.sendrecv(message)
+		reply = KRB5ChangePassword.from_bytes(response)
+		privresponse = reply.parse_reply(subkey_cipher, subkey)
+		return privresponse
